# ruff: noqa: F401

from gaphor.UML.actions import (
    actionseditors,
    actionspropertypages,
    activitymove,
    activitypropertypage,
    partitionpage,
)
from gaphor.UML.classes import (
    associationpropertypages,
    classeseditors,
    classespropertypages,
    dependencypropertypages,
    enumerationpropertypages,
)
from gaphor.UML.interactions import interactionspropertypages
from gaphor.UML.profiles import metaclasspropertypage, stereotypepropertypages
from gaphor.UML.states import dropzone, propertypages
